
/*
 * Simple tx transmitter from
 * https://github.com/pu2clr/Small
 * 
 * For more info:
 * 110010100.neocities/tx-am.html
 * 
 * Requeriments:
 * 
 * 1. Rotary.h library
 * 2. LetrasLCD.h Library
 * 3. si5351.h Library
*/
#include <si5351.h>
#include <Wire.h>
#include "Rotary.h"
#include <LiquidCrystal_I2C.h>
#include "letrasLCD.h"
LiquidCrystal_I2C lcd(0x27, 16, 2); // I2C address 0x27, 16 column and 2 rows


// Enconder PINs
#define ENCODER_PIN_A 2 // Encoder pin A
#define ENCODER_PIN_B 3 // Encoder pin B
#define SWITCH_CMD 4   // ENCODER button or regular push button

// Valores iniciales de paso, banda, frecuencia y velocidad.
#define CMD_STEP     0
#define CMD_FAVORITE 1
#define CMD_FREQ 2

// Change this value bellow  (CORRECTION_FACTOR) to 0 if you do not know the correction factor of your Si5351A.
#define CORRECTION_FACTOR 100000 // See how to calibrate your Si5351A (0 if you do not want).

// VFO range for this project is 3000 KHz to 30000 KHz (3MHz to 30MHz).
//#define MIN_VFO 3 000 000 00LLU     // VFO min. frequency 3Mhz
#define MIN_VFO 10000000LLU
#define MAX_VFO 3000000000LLU    // VFO max. frequency 30MHz

// Struct for step
typedef struct
{
  char *name; // step label: 50Hz, 10Hz, 500Hz and 100KHz
  long value; // Frequency value (unit 0.01Hz See documentation) to increment or decrement
} Step;

// Steps database. You can change the Steps and numbers of steps here if you need.
Step step[] = {
  {(char *) "100Hz ", 10000}, // Minimum Frequency step (incremente or decrement) 100Hz
  {(char *) "500Hz ", 50000},
  {(char *) "1KHz  ", 100000},
  {(char *) "5KHz  ", 500000},
  {(char *) "10KHz ", 1000000},
  {(char *) "50KHz ", 5000000},
  {(char *) "500KHz", 50000000}
}; // Maximum frequency step 500KHz

// Calculate the index of last position of step[] array
const int lastStepVFO = (sizeof step / sizeof(Step)) - 1;
int currentStep = 4;


// Your favotite frequencies
uint64_t favorite[] = {  100000000LLU, 400000000LLU, 500000000LLU,
                        600000000LLU, 1000000000LLU, 1100000000LLU,
                        1200000000LLU, 1500000000LLU, 1700000000LLU,
                        2700000000LLU
                      };

const int lastFavorite = (sizeof favorite / sizeof(uint64_t) ) - 1;
int currentFavorite = 0;
int currentCommand = CMD_FREQ;
bool isFreqChanged = true;

// Encoder controller
Rotary encoder = Rotary(ENCODER_PIN_A, ENCODER_PIN_B);

// The Si5351 instance.
Si5351 si5351;

uint64_t vfoFreq;
uint64_t vfoFreqOffset;
uint64_t vfoLastValue;
uint64_t offSet = 70000; // Establece el offset del oscilador local en menos 700hz
double vfo;

// Encoder control variables
volatile int encoderCount = 0;

void setup()
{
    
  // Encoder pins
  pinMode(ENCODER_PIN_A, INPUT_PULLUP);
  pinMode(ENCODER_PIN_B, INPUT_PULLUP);
  pinMode(SWITCH_CMD, INPUT_PULLUP);

  // Iniciando el LCD
  lcd.init();
  lcd.backlight();
  lcd.createChar(0, LetraRcursor);
  lcd.createChar(1, LetraBcursor);
  lcd.createChar(2, LetraPcursor);
  lcd.createChar(3, LetraScursor);
  lcd.createChar(4, LetraP);
  lcd.createChar(5, LetraT);
  lcd.createChar(6, ili);
  lcd.setCursor(0,0);
  lcd.print("Simple Am");
  lcd.setCursor(0,1);
  lcd.print("Transmisor");
  // Imprimimos marca ili
  lcd.setCursor(15,0);
  lcd.write(6); 
  delay(3000);
  lcd.clear();
 
  // Ponemos en frecuencia el oscilador local
  vfoFreq = favorite[currentFavorite];
  vfoFreqOffset = vfoFreq - offSet; // currentStep * direction;

  
  // Initiating the Signal Generator (si5351)
  si5351.init(SI5351_CRYSTAL_LOAD_8PF, 0, 0);
  // Adjusting the frequency (see how to calibrate the Si5351 - example si5351_calibration.ino)
  si5351.set_correction(CORRECTION_FACTOR, SI5351_PLL_INPUT_XO);
  si5351.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);
  //si5351.drive_strength(SI5351_CLK0,SI5351_DRIVE_4MA); //La potencia puede ser 2MA, 4MA, 6MA o 8MA
  si5351.set_freq(vfoFreq, SI5351_CLK0); // Start CLK0 (VFO)
  //si5351.set_freq(vfoFreqOffset, SI5351_CLK2); // Start CLK2 (VFO con offset)

  // Disable CLK 1 and 2 outputs
  si5351.output_enable(SI5351_CLK1, 0);
  si5351.output_enable(SI5351_CLK2, 0);
  si5351.update_status();
  showStatus();
  delay(500);
  // Encoder interrupt
  attachInterrupt(digitalPinToInterrupt(ENCODER_PIN_A), rotaryEncoder, CHANGE);
  attachInterrupt(digitalPinToInterrupt(ENCODER_PIN_B), rotaryEncoder, CHANGE);
}
// Use Rotary.h and  Rotary.cpp implementation to process encoder via interrupt
void rotaryEncoder()
{ // rotary encoder events
  uint8_t encoderStatus = encoder.process();
  if (encoderStatus)
  {
    if (encoderStatus == DIR_CW)
    {
      encoderCount = 1;
    }
    else
    {
      encoderCount = -1;
    }
  }
}


// Show Signal Generator Information
// Verificar setCursor() em https://github.com/greiman/SSD1306Ascii/issues/53
void showStatus()
{
  vfo = vfoFreq / 100000.00;
  
  switch (currentCommand){
    case CMD_STEP:  
    lcd.setCursor(2,0);
    lcd.print(" ");
    
    lcd.setCursor(1,1);
    lcd.write(2);
    lcd.print(step[currentStep].name);
    break;
  
    case CMD_FAVORITE:
    lcd.setCursor(1,1);
    lcd.print(" ");
 
    lcd.setCursor(2,0);
    lcd.write(1);
    lcd.print(vfo);
    lcd.print("khz ");
    break;

    default:
    lcd.setCursor(15,1);
    lcd.write(6);
    
    lcd.setCursor(1,1);
    lcd.print(" ");
    lcd.print(step[currentStep].name);
       
    lcd.setCursor(2,0);
    lcd.write(5);
    lcd.print(vfo);
    lcd.print("khz ");
    break;    
  }
 
}

// Change the frequency (increment or decrement)
// direction parameter is 1 (clockwise) or -1 (counter-clockwise)
void changeFreq(int direction)
{
  vfoFreq += step[currentStep].value * direction; // currentStep * direction;
  vfoFreqOffset = vfoFreq - offSet;
  // Check the VFO limits
  if (vfoFreq > MAX_VFO )
    vfoFreq = MIN_VFO;
  else if (vfoFreq < MIN_VFO)
    vfoFreq = MAX_VFO;
  isFreqChanged = true;
}

void doCommandUp() {
  if ( currentCommand == CMD_STEP )
    currentStep = (currentStep < lastStepVFO) ? (currentStep + 1) : 0;
  else {
    currentFavorite = (currentFavorite < lastFavorite) ? (currentFavorite + 1) : 0;
    vfoFreq = favorite[currentFavorite];
    isFreqChanged = true;
  }

  delay(200);
}

void doCommandDown() {
  if ( currentCommand == CMD_STEP )
    currentStep = (currentStep > 0) ? (currentStep - 1) : lastStepVFO;
  else {
    currentFavorite = (currentFavorite > 0) ? (currentFavorite - 1) : lastFavorite;
    vfoFreq = favorite[currentFavorite];
    isFreqChanged = true;
  }
  delay(200);
}

void loop()
{

 if (currentCommand == CMD_STEP) {
    if (encoderCount == 1){ doCommandUp(); encoderCount = 0; }
    if (encoderCount == -1){ doCommandDown(); encoderCount = 0; }
    showStatus();
    if (digitalRead(SWITCH_CMD) ==  LOW) { currentCommand = CMD_FAVORITE; delay(250); }
 }
 else if (currentCommand == CMD_FAVORITE) {
    if (encoderCount == 1){ doCommandUp(); encoderCount = 0; }
    if (encoderCount == -1){ doCommandDown(); encoderCount = 0; }
    showStatus();
    if (digitalRead(SWITCH_CMD) ==  LOW) { currentCommand = CMD_FREQ; delay(250); showStatus();}
 }
 else {
      if (encoderCount != 0) { 
            if (encoderCount == 1) { changeFreq(1); }
            else { changeFreq(-1); }
            encoderCount = 0;
      }
      if (isFreqChanged) {
          vfoFreqOffset = vfoFreq - offSet; 
          si5351.set_freq(vfoFreqOffset, SI5351_CLK0);
          isFreqChanged = false;
          showStatus();
      } 
    
    if (digitalRead(SWITCH_CMD) ==  LOW) { currentCommand = CMD_STEP; delay(250);}  
  }

}
